# -*- coding: utf-8 -*-
"""
Created on Tue Jun 23 14:54:02 2015

@author: af26

Takes connectivity graphs for several years and combines them into a single 
graph. Two edge weights are given: one is a sum of the weights in the 
years, the other is the number of years when the connection exists.
"""

import networkx as nx
import numpy as np
import matplotlib.pyplot as plt
import csv
import pandas as pd
from scipy import signal
import scipy.stats as stats

def read_nao_monthly(filename):
    year = []
    month = []
    nao = []
    for line in infile:
        wordlist = line.split()
        year.append(int(wordlist[0]))
        month.append(int(wordlist[1]))
        nao.append(float(wordlist[2]))
    return year, month, nao
    
filename = ('C:/Users/af26/NAO/nao_monthly.txt')
infile = open(filename,'r')

yr, mon, nao = read_nao_monthly(infile)

startyear = 1965
endyear = 2004

folder = 'E:/af26/LarvalDispersalResults/polcoms1965to2005/CrossingPlots/'
run = 'doublelife'

years = range(startyear,endyear+1)
print_years = list(years)
print_years.insert(0,'years')

nao_jan = np.array([nao[i] for i in range(len(nao)) 
            if mon[i] == 1 and yr[i] >= startyear and yr[i] <= endyear])
nao_feb = np.array([nao[i] for i in range(len(nao)) 
            if mon[i] == 2 and yr[i] >= startyear and yr[i] <= endyear])
nao_mar = np.array([nao[i] for i in range(len(nao)) 
            if mon[i] == 3 and yr[i] >= startyear and yr[i] <= endyear])
nao_apr = np.array([nao[i] for i in range(len(nao)) 
            if mon[i] == 4 and yr[i] >= startyear and yr[i] <= endyear])
nao_may = np.array([nao[i] for i in range(len(nao)) 
            if mon[i] == 5 and yr[i] >= startyear and yr[i] <= endyear])

nao_meanfm = (nao_mar + nao_apr + nao_may)/3.0

release_site = ['Anton Dohrn Seamount','Darwin Mounds',
                'East Rockall Bank','Faroe-Shetland Sponge Belt',
                'Hatton Bank','North West Rockall Bank',
                'Rosemary Bank Seamount','South-East Rockall Bank SAC',
                'Wyville Thomson Ridge','East Mingulay',
                'Central Fladen','Norwegian Boundary Sediment Plain']
                
sections = ['A1','A2','A3','A4','A5',
            'B1','B2','B3','B4',
            'C1',
            'S1','S2'
            ]

with open(folder + 'crossings2_larvalnumbers_' + run + '.csv', 'wb') as f:
    writer = csv.writer(f)
    writer.writerow(print_years)
    x = nao_jan.tolist()
    x.insert(0,'NAO Jan')
    writer.writerow(x)
    x = nao_feb.tolist()
    x.insert(0,'NAO Feb')
    writer.writerow(x)
    x = nao_mar.tolist()
    x.insert(0,'NAO Mar')
    writer.writerow(x)
    x.insert(0,'NAO Apr')
    writer.writerow(x)
    x = nao_apr.tolist()
    x.insert(0,'NAO May')
    writer.writerow(x)
    x = nao_may.tolist()

    print_header1 = ['Section',
                    'A1','','A2','','A3','','A4','','A5','',
                    'B1','','B2','','B3','','B4','',
                    'C1','',
                    'S1','','S2',''
                     ]    
    print_header2 = ['Correlation',
                    'r','p','r','p','r','p','r','p','r','p',
                    'r','p','r','p','r','p','r','p',
                    'r','p',
                    'r','p','r','p'
                    ]    
    
    with open(folder + 'crossings2_correlations_nao_mam_' + run + '.csv', 'wb') as g:
        writer2 = csv.writer(g)
        writer2.writerow(print_header1)
        writer2.writerow(print_header2)
        
        nint = len(years)
        
        for site in release_site:
            print site
            sections_dict = {}
            for section in sections:
                sections_dict[section] = np.zeros(nint)
                       
            for year in years:
                i = year - startyear
                graph_file = ('E:/af26/LarvalDispersalResults/' + 
                                'polcoms' + str(year) + 
                                '/Run_1000_'+ run +'/Crossingdata/' + 
                                site + '_crosses2.graphml')
                infile = open(graph_file,'r')                
                G = nx.read_graphml(infile)
                infile.close()
                   
                for u,v,data in G.edges_iter(data=True):

                    sections_dict[v][i] = data['nstayed']
                    
                                               
            writer.writerow([site])
            for section in sections:
                x = sections_dict[section].tolist()
                x.insert(0,section)
                writer.writerow(x)
            
            cor = [site]
            for section in sections:
                x = stats.pearsonr(nao_meanfm,sections_dict[section])
                cor.append(x[0])
                cor.append(x[1])

            
            writer2.writerow(cor)
            
            print stats.pearsonr(nao_meanfm,sections_dict['S1'])
    
